% Simulating the dumped Brusselator by the Finite Difference Method(a time march)
% Numerical scheme used is a first order upwind in time and a second order
...central difference in space (both Implicit and Explicit)
%%
clear; clc; close all;
f  = @(u,v,alfa,beta)(alfa+u.^2.*v -(beta+1)*u);  % reaction 1
g  = @(u,v,beta)(beta*u-u.^2.*v);                 % reaction 2
%Specifying Parameters
N    = 50;              %Number of steps in space(x)
M    = 40;              %Number of time steps 
dt   = 0.05;            %Width of each time step
L    = 1;               %Space length
dx   = L/(N-1);         %Width of space step
x    = 0:dx:L;          %Range of x (0,L) and specifying the grid points
D    = 1/50;            %Diffusion coefficient
alfa = 1;               %Reaction rate
beta = 3;               %Reaction rate
r    = D*dt/(dx*dx);    %Stability criterion (0<=r<=0.5)
UL   = 1;               %Left Dirichlet B.C for u
UR   = 1;               %Right Dirichlet B.C for u
VL   = 3;               %Left Dirichlet B.C for v
VR   = 3;               %Right Dirichlet B.C for v
%%
%Initial Conditions
u = 1 + sin(2*pi*x); u=u';
v = 3*ones(size(x)); v = v';
%%
%B.C vector
bcu = zeros(N-2,1); bcv = zeros(N-2,1);
bcu(1) = r*UL; bcu(N-2) = r*UR;  
bcv(1) = r*VL; bcv(N-2) = r*VR;  
%Calculating the coefficient matrix 
E = sparse(2:N-2,1:N-3,1,N-2,N-2);
A = E+E' - 2*speye(N-2);      
B = speye(N-2) - r*A; % matrix for the implicit scheme
%%
%Calculating the velocity profile for each time step
for m = 0:M
    um = u; vm = v;
    h = plot(x,u,x,v);       %plotting the concentration profile
    axis([0 L 0 4])
    title({['Brusselator with D =',num2str(D),', \alpha = ',num2str(alfa),', \beta = ',num2str(beta),' and r = ',num2str(r)];['time(\itt) = ',num2str(dt*m)]})
    xlabel('Spatial co-ordinate (x)')
    ylabel('Concentration profile')
    legend('u','v')
    drawnow; 
    refreshdata(h)
    %Implicit solution
    U = um; U(1) = []; U(end) = [];
    V = vm; V(1) = []; V(end) = [];
    %Implicit solution
    U = B\(U + bcu + dt*f(U,V,alfa,beta));
    V = B\(V + bcv + dt*g(U,V,beta));
    %Explicit solution
    %U = U + r*A*U + bcu + dt*f(U,V,alfa,beta);
    %V = V + r*A*V + bcv + dt*g(U,V,beta);
    %Boundary conditions
    u = [UL;U;UR];                    
    v = [VL;V;VR];                     
end