% Simulating the 1-D Diffusion equation (Fourier's equation) by the
...Finite Difference Method(a time march)
% Numerical scheme used is a first order upwind in time and a second order
...central difference in space (Implicit and Explicit)
%%
clear; clc; close all;
%Specifying Parameters
N   = 50;             %Number of steps in space(x)
M   = 200;            %Number of time steps 
dt  = 0.05;            %Width of each time step
L   = 2;              %Space length
dx  = L/(N-1);        %Width of space step
x   = 0:dx:L;         %Range of x (0,L) and specifying the grid points
u   = zeros(N,1);     %Concentration u
um  = zeros(N,1);     %Concentration u^m (previous time level)
D   = 1;           %Diffusion coefficient
r   = D*dt/(dx*dx);   %Stability criterion (0<=r<=0.5)
UL  = 0;              %Left Dirichlet B.C
UR  = 0;              %Right Dirichlet B.C 
UnL = 0;              %Left Neumann B.C (du/dn=UnL) 
UnR = 0;              %Right Neumann B.C (du/dn=UnR) 

%%
%Initial Conditions
for i = 1:N
        Lv = 100; u(i) = exp(-(i-N/2)^2/Lv);
end

%%
%B.C vector
bc = zeros(N-2,1);
%bc(1) = r*UL; bc(N-2) = r*UR;  %Dirichlet B.Cs
bc(1) = -r*UnL; bc(N-2) = r*UnR;  %Neumann B.Cs
%Calculating the coefficient matrix for the implicit scheme
E = sparse(2:N-2,1:N-3,1,N-2,N-2);
A = E+E' - 2*speye(N-2);      
A(1,1) = -1; A(N-2,N-2) = -1; %Neumann B.Cs (comment for Dirichlet B.Cs)
B = speye(N-2) - r*A;

%%
%Calculating the concentration for each time step
for m = 0:M
    um = u;
    h = plot(x,u);       %plotting the concentration profile     
    axis([0 L 0 3])
    title({['1-D Diffusion with D =',num2str(D),' and r = ',num2str(r)];['time(\itt) = ',num2str(dt*m)]})
    xlabel('Spatial co-ordinate (x)')
    ylabel('Concentration profile (u)')
    drawnow; 
    refreshdata(h)
    U = um; U(1) = [];U(end)=[];
    U = B\(U + bc);          %Implicit method
    %U = U + r*A*U + bc;      %Explicit method
    %u = [UL;U;UR];                     %Dirichlet
    u = [U(1)-UnL*dx;U;U(end)+UnR*dx]; %Neumann
end