% Simulating the 1-D Diffusion equation (Fourier's equation) by the Finite
... with Difference Method (Dirichlet boundary conditions)
% Numerical scheme used is a first order explicit upwind in time and a 
... second order central difference in space
%%
clear; clc; close all;
%Specifying Parameters
N   = 50;              %Number of steps in space(x)
M   = 200;             %Number of time steps 
dt  = 0.05;            %Width of each time step
L   = 2;               %Space length
dx  = L/(N-1);         %Width of space step
x   = 0:dx:L;          %Range of x (0,L) and specifying the grid points
u   = zeros(N,1);      %Concentration u
um  = zeros(N,1);      %Concentration u^m (previous time level)
D   = 0.01;            %Diffusion coefficient
r   = D*dt/(dx*dx);    %Stability criterion (0<=r<=0.5)
UL  = 0;               %Left Dirichlet B.C
UR  = 0;               %Right Dirichlet B.C 
%%
%Initial Conditions: a square wave
for i=1:N
    if ((0.8 <= x(i)) && (x(i) <= 1.2))
        u(i) = 1;
    else
        u(i) = 0;
    end
end
%%
%Calculating the concentration for each time step
i = 2:N-1;
for m = 0:M
    um = u;
    h = plot(x,u);       %plotting the concentration profile
    axis([0 L 0 1.5])
    title({['1-D Diffusion with D =',num2str(D),' and r = ',num2str(r)];['time(\itt) = ',num2str(dt*m)]})
    xlabel('Spatial co-ordinate (x)')
    ylabel('Concentration profile (u)')
    drawnow; 
    refreshdata(h)
    %Explicit method with F.D. in time and C.D. in space
    u(1) = UL; u(N) = UR;
    u(i) = um(i) + r*(um(i+1) - 2*um(i) + um(i-1));
end