% Simulating the 1-D Diffusion equation (Fourier's equation) by the
...Finite Difference Method(a time march)
% Numerical scheme used is a first order upwind in time and a second order
...central difference in space (Implicit and Explicit)
%%
clear; clc; close all;
f   = @(u,mu)(mu*u.*(1-u));  % reaction
%Specifying Parameters
N   = 100;             %Number of steps in space(x)
M   = 100;             %Nmber of time steps 
dt  = 0.1;             %Width of each time step
L   = 10;              %Space length
dx  = L/(N-1);         %Width of space step
x   = 0:dx:L;          %Range of x (0,L) and specifying the grid points
u   = zeros(N,1);      %Concentration u
um  = zeros(N,1);      %Concentration u^m (previous time level)
D   = 0.5;               %Diffusion coefficient
mu  = 0.1;            %Reaction rate
r   = D*dt/(dx*dx);    %Stability criterion (0<=r<=0.5)
UL  = 0;               %Left Dirichlet B.C
UR  = 0;               %Right Dirichlet B.C 
UnL = 0;               %Left Neumann B.C (du/dn=UnL) 
UnR = 0;               %Right Neumann B.C (du/dn=UnR) 
%%
%Initial Conditions: a square wave
for i=1:N
    if ((4.8 <= x(i)) && (x(i) <= 5.2))
        u(i) = 3;
    else
        u(i) = 0;
    end
end
%%
%B.C vector
bc = zeros(N-2,1);
bc(1) = r*UL; bc(N-2) = r*UR;  %Dirichlet B.Cs
%bc(1) = -UnL*D*dt/dx; bc(N-2) = UnR*D*dt/dx;  %Neumann B.Cs
%Calculating the coefficient matrix 
E = sparse(2:N-2,1:N-3,1,N-2,N-2);
A = E+E' - 2*speye(N-2);      
%A(1,1) = -1; A(N-2,N-2) = -1; %Neumann B.Cs (comment for Dirichlet B.Cs)
B = speye(N-2) - r*A; % Matrix for the implicit scheme
%%
%Calculating the velocity profile for each time step
i = 2:N-1;
for m = 0:M
    um = u;
    h = plot(x,u);       %plotting the concentration profile
    axis([0 L 0 4])
    title({['Fisher with D =',num2str(D),', \mu = ',num2str(mu),', L^* = ',num2str(pi*sqrt(D/mu)),' and r = ',num2str(r)];['time(\itt) = ',num2str(dt*m)]})
    xlabel('Spatial co-ordinate (x)')
    ylabel('Concentration profile (u)')
    drawnow; 
    refreshdata(h)
    U = um; U(1) = []; U(end) = [];
    U = B\(U + bc + dt*f(U,mu));        %Implicit solution
    %U = U + r*A*U + bc + dt*f(U,mu);    %Explicit solution
    u = [UL;U;UR];                     %Dirichlet
    %u = [U(1)-UnL*dx;U;U(end)+UnR*dx]; %Neumann
end